@@ -374,7 +374,12 @@ class AzureBlobFileSystem(AsyncFileSystem):
             and self.sas_token is None
             and self.client_id is not None
         ):
-            self.credential = self._get_credential_from_service_principal()
+            (
+                self.credential,
+                self.sync_credential,
+            ) = self._get_credential_from_service_principal()
+        else:
+            self.sync_credential = None
         self.do_connect()
 
     @classmethod
@@ -412,16 +417,26 @@ class AzureBlobFileSystem(AsyncFileSystem):
 
         Returns
         -------
-        Credential
+        Tuple of (Async Credential, Sync Credential).
         """
         from azure.identity import ClientSecretCredential
+        from azure.identity.aio import (
+            ClientSecretCredential as AIOClientSecretCredential,
+        )
 
-        sp_token = ClientSecretCredential(
+        async_credential = AIOClientSecretCredential(
             tenant_id=self.tenant_id,
             client_id=self.client_id,
             client_secret=self.client_secret,
         )
-        return sp_token
+
+        sync_credential = ClientSecretCredential(
+            tenant_id=self.tenant_id,
+            client_id=self.client_id,
+            client_secret=self.client_secret,
+        )
+
+        return (async_credential, sync_credential)
 
     def do_connect(self):
         """Connect to the BlobServiceClient, using user-specified connection details.
@@ -917,20 +932,23 @@ class AzureBlobFileSystem(AsyncFileSystem):
                 container_client = self.service_client.get_container_client(
                     container=container_name
                 )
-                await container_client.upload_blob(name=path, data="")
+                await container_client.upload_blob(name=path, data="", overwrite=True)
             else:
                 ## everything else
-                raise RuntimeError(f"Cannot create {container_name}{delimiter}{path}.")
+                raise FileExistsError(
+                    f"Cannot overwrite existing Azure container -- {container_name} already exists."
+                )
         else:
             try:
                 if (container_name_as_dir in _containers) and path:
                     container_client = self.service_client.get_container_client(
                         container=container_name
                     )
-                    await container_client.upload_blob(name=path, data="")
+                    await container_client.upload_blob(
+                        name=path, data="", overwrite=False
+                    )
             except ResourceExistsError:
-                pass
-        # _ = await self._ls("", invalidate_cache=True)
+                raise FileExistsError(f"{container_name_as_dir}{path} already exists!!")
         self.invalidate_cache(self._parent(path))
 
     def rm(self, path, recursive=False, maxdepth=None, **kwargs):
@@ -1141,22 +1159,33 @@ class AzureBlobFileSystem(AsyncFileSystem):
             raise FileNotFoundError
         return list(sorted(out))
 
-    async def _put_file(self, lpath, rpath, delimiter="/", **kwargws):
-        """ Copy single file to remote """
+    async def _put_file(self, lpath, rpath, delimiter="/", overwrite=False, **kwargws):
+        """
+        Copy single file to remote
+
+        :param lpath: Path to local file
+        :param rpath: Path to remote file
+        :param delimitier: Filepath delimiter
+        :param overwrite: Boolean (False).  If True, overwrite the existing file present
+        """
+
         container_name, path = self.split_path(rpath, delimiter=delimiter)
         cc = self.service_client.get_container_client(container_name)
         bc = cc.get_blob_client(blob=path)
-        try:
-            with open(lpath, "rb") as f1:
-                await bc.upload_blob(f1)
-            # self.invalidate_cache(self._parent(path))
-            self.invalidate_cache()
-        except ResourceExistsError:
-            raise FileExistsError("File already exists!!")
-        except ResourceNotFoundError:
-            await cc.create_container()
-            await self._put_file(lpath, rpath, delimiter)
-            self.invalidate_cache()
+        if os.path.isdir(lpath):
+            self.makedirs(rpath, exist_ok=True)
+        else:
+            try:
+                with open(lpath, "rb") as f1:
+                    await bc.upload_blob(f1, overwrite=overwrite)
+                self.invalidate_cache()
+            except ResourceExistsError:
+                raise FileExistsError("File already exists!!")
+            except ResourceNotFoundError:
+                if not await self._exists(container_name):
+                    raise FileNotFoundError("Container does not exist.")
+                await self._put_file(lpath, rpath, delimiter, overwrite)
+                self.invalidate_cache()
 
     put_file = sync_wrapper(_put_file)
 
@@ -1370,9 +1399,9 @@ class AzureBlobFile(io.IOBase):
             self.fs.account_url: str = (
                 f"https://{self.fs.account_name}.blob.core.windows.net"
             )
-            if self.fs.credential is not None:
+            if self.fs.sync_credential is not None:
                 self.container_client = BlobServiceClient(
-                    account_url=self.fs.account_url, credential=self.fs.credential
+                    account_url=self.fs.account_url, credential=self.fs.sync_credential
                 ).get_container_client(self.container_name)
             elif self.fs.connection_string is not None:
                 self.container_client = BlobServiceClient.from_connection_string(
